/*

NSString-NIRAdditions.m

DirectionService - controll writting direction
Copyright (C) 2003 Nir Soffer

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details: http://www.gnu.org/licenses/gpl.html

*/

#import <CoreFoundation/CoreFoundation.h>
#import "NSString-NIRAdditions.h"
#import "NSAttributedString-NIRAdditions.h"

@implementation NSString (NSStringNIRAdditions)

+ (id)stringWithContentsOfFile:(NSString *)absolutePath
                      encoding:(unsigned)encoding; {
    // Read file into NSData then create string using encoding
    NSData *data = [[NSData alloc] initWithContentsOfFile:absolutePath];
    id instance = [[[self class] alloc] initWithData:data
                                            encoding:encoding];
    [data release];
    return [instance autorelease];
}

+ (id)stringWithCharacter:(unichar)aUnichar
    /* utility method to create strings from Unicode character */
{
    return [[self class] stringWithCharacters:&aUnichar length:1];
}

- (BOOL)hasLines
{
    // test if a string has paragrphs or it is just some characters
    
    unsigned end, contentsEnd;

    [self getLineStart:nil end:&end contentsEnd:&contentsEnd forRange:NSMakeRange(0,0)];
    return (contentsEnd != end);
}

- (NSRange)range; {
    // Return the full range of the reciver
    // A shortcut instead of writing this again and again
    return NSMakeRange(0, [self length]);
}

@end


@implementation NSMutableString (NSMutableStringNIRAdditions)

- (unsigned)deleteCharactersFromSet:(NSCharacterSet *)charSet
                            options:(unsigned int)mask
                              range:(NSRange)searchRange {
    /* Delete all accurences of characters from charSet.

    options -- options of rangeOfCharacterFromSet:options:range:
    Return the number of deleted characters.
    */

    NSRange foundRange = NSMakeRange(0, 0);
    unsigned deleted = 0;

    while (foundRange.location != NSNotFound) {
        foundRange = [self rangeOfCharacterFromSet:charSet
                                           options:NSBackwardsSearch | mask
                                             range:searchRange];
        if (foundRange.location != NSNotFound) {
            [self deleteCharactersInRange:foundRange];
            deleted += foundRange.length;
            searchRange.length = foundRange.location;
        }
    }
    return deleted;
}

- (unsigned)deleteCharactersFromSet:(NSCharacterSet *)charSet {
    /* Invoke deleteCharactersFromSet:options:range with no options
    and self range.
    */
    return [self deleteCharactersFromSet:charSet
                                 options:NULL
                                   range:NSMakeRange(0, [self length])];
}

@end

