//
//  NIRHebrewService.m
//  Hebrew Service
//
//  Created by Nir Soffer on Sat Dec 06 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

// Import core foundation for cp-1255 encoding.
#import <CoreFoundation/CoreFoundation.h>

#import "NIRHebrewService.h"
#import "NSString-FriBidiAdditions.h"
#import "NSPasteboard-NIRAdditions.h"


@implementation NIRHebrewService

- (void)convertFromHebrewWindows:(NSPasteboard *)pboard
                        userData:(NSString *)data
                           error:(NSString **)error; {
    /* Convert the selection from ISO Latin 1 */

    // Get string from the pasteboard
    NSString *selection = [self stringFromPasteboard:pboard];
    if (selection == nil) {
        *error = @"Could not get selection from the pasteboard";
        return;
    }

    /*
     The string we get is Hebrew string using cp1255 encoding, but it was 
     decoded as iso-8859-1 string. To fix it, we first encode it back to 
     iso-8859-1, then decode it again as cp1255.
     */
    
    // Try to encode using iso-8859-1, replacing what can not be converted
    NSData *tmp = [selection dataUsingEncoding:NSISOLatin1StringEncoding
                          allowLossyConversion:YES];
    if (tmp == nil) {
        *error = @"Could not encode selection as iso-8859-1 string";
        return;
    }
    
    // Try to decode using cp1255
    UInt32 encoding = 
        CFStringConvertEncodingToNSStringEncoding(kCFStringEncodingWindowsHebrew);
    NSString *converted = [[NSString alloc] initWithData:tmp encoding:encoding];
    if (converted == nil) {
        *error = @"Could not decode selection as Hebrew (Windows)";
        return;
    }
    [converted autorelease];
    
    // Return string
    if (! [pboard putObject:converted forType:NSStringPboardType]) {
        *error = @"Could not put converted string on the pasteboard";
    }
}

- (void)reverseSelection:(NSPasteboard *)pboard
                userData:(NSString *)data
                   error:(NSString **)error; {
    /* Reverse the selection using Bidi algorithm */

    // Get string from the pasteboard
    NSString *selection = [self stringFromPasteboard:pboard];
    if (selection == nil) {
        *error = @"Could not get selection from the pasteboard";
        return;
    }

    // Reverse
    NSString *reversed = [selection stringByOrderingVisually];
    if (reversed == nil) {
        *error = @"Could not reverse selection";
        return;
    }

    // Return string
    if (! [pboard putObject:reversed forType:NSStringPboardType]) {
        *error = @"Could not put reversed string on the pasteboard";
    }
}

- (NSString *)stringFromPasteboard:(NSPasteboard *)pboard; {
    /* get string from pasteboard */

    NSArray *types = [[NSArray alloc] initWithObjects:NSStringPboardType, nil];
    NSString *type = [pboard availableTypeFromArray:types];
    [types release];
    if (type != nil) {
        return [pboard stringForType:type];
    }
    return nil;
}

@end
